package json;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import type2.Type2GrammarGenerator;
import type3.Type3GrammarGenerator;

public class GrammarToJSON {

	Type3GrammarGenerator gen3 = new Type3GrammarGenerator();
	Type2GrammarGenerator gen2 = new Type2GrammarGenerator();

	private String createJSONGrammar(String[] vars, int alphaSize, Map<String, List<List<String>>> rules,
			String start) {
		JSONObject grammarJSON = new JSONObject();
		JSONArray varsJSON = new JSONArray();
		for (String v : vars) {
			varsJSON.add(new JSONString(v));
		}
		JSONArray termsJSON = new JSONArray();
		for (int i = 97; i < 97 + alphaSize; i++) {
			termsJSON.add(new JSONString(((char) i) + ""));
		}
		JSONArray rulesJSON = new JSONArray();
		int ruleNo = 1;
		for (Entry<String, List<List<String>>> rule : rules.entrySet()) {
			String premise = rule.getKey();
			for (List<String> conclusio : rule.getValue()) {
				JSONObject ruleJSON = new JSONObject();
				JSONArray conclusioJSON = new JSONArray();
				for (String c : conclusio) {
					conclusioJSON.add(new JSONString(c));
				}
				ruleJSON.put("ruleNo", new JSONNumber(ruleNo++));
				ruleJSON.put("premise", new JSONString(premise));
				ruleJSON.put("conclusio", conclusioJSON);
				rulesJSON.add(ruleJSON);
			}
		}
		grammarJSON.put("variables", varsJSON);
		grammarJSON.put("terminals", termsJSON);
		grammarJSON.put("rules", rulesJSON);
		grammarJSON.put("start", new JSONString(start));
		return grammarJSON.toString();
	}

	public List<String> createJSONGrammars(String[] vars, int alphaSize, int maxConclusios, int maxConclusioSize,
			String start, int type, int mode, int n) {
		if (type == 3) {
			return createJSONObjectType3(vars, alphaSize, maxConclusios, start, mode, n);
		} else if (type == 2) {
			return createJSONObjectType2(vars, alphaSize, maxConclusios, maxConclusioSize, start, mode, n);
		} else {
			throw new IllegalArgumentException("Argument 'type' has to either be 2 or 3.");
		}
	}

	private List<String> createJSONObjectType3(String[] vars, int alphaSize, int maxConclusios, String start, int mode,
			int n) {
		List<Map<String, List<List<String>>>> ruleSets = new LinkedList<>();
		switch (mode) {
		case 0:
			ruleSets = gen3.genType3GrammarRuleSets0(vars, alphaSize, maxConclusios, start, n);
			break;
		case 1:
			ruleSets = gen3.genType3GrammarRuleSets1(vars, alphaSize, maxConclusios, start, n);
			break;
		case 2:
			ruleSets = gen3.genType3GrammarRuleSets2(vars, alphaSize, maxConclusios, start, n);
			break;
		case 3:
			ruleSets = gen3.genType3GrammarRuleSets3(vars, alphaSize, maxConclusios, start, n);
			break;
		default:
			break;
		}
		List<String> jsonGrammars = new LinkedList<>();
		for (Map<String, List<List<String>>> ruleSet : ruleSets) {
			jsonGrammars.add(createJSONGrammar(vars, alphaSize, ruleSet, start));
		}
		return jsonGrammars;
	}

	private List<String> createJSONObjectType2(String[] vars, int alphaSize, int maxConclusios, int maxConclusioSize,
			String start, int mode, int n) {
		List<Map<String, List<List<String>>>> ruleSets = new LinkedList<>();
		switch (mode) {
		case 0:
			ruleSets = gen2.genType2GrammarRuleSets0(vars, alphaSize, maxConclusios, maxConclusioSize, n);
			break;
		case 1:
			ruleSets = gen2.genType2GrammarRuleSets1(vars, alphaSize, maxConclusios, maxConclusioSize, start, n);
			break;
		case 2:
			ruleSets = gen2.genType2GrammarRuleSets2(vars, alphaSize, maxConclusios, maxConclusioSize, n);
			break;
		case 3:
			ruleSets = gen2.genType2GrammarRuleSets3(vars, alphaSize, maxConclusios, maxConclusioSize, start, n);
			break;
		default:
			break;
		}
		List<String> jsonGrammars = new LinkedList<>();
		for (Map<String, List<List<String>>> ruleSet : ruleSets) {
			jsonGrammars.add(createJSONGrammar(vars, alphaSize, ruleSet, start));
		}
		return jsonGrammars;
	}

	public static void main(String[] args) {

	}

}
